"""
Tests for the theme auth hooks
"""

# Standard Library
from unittest.mock import patch

# Django
from django.test import TestCase

# AA Templates: Terra Nanotech
from tnnt_templates.theme.terra_nanotech import auth_hooks


class TerraNanotechThemeHookTests(TestCase):
    """
    Test the theme hook
    """

    def setUp(self):
        """
        Set up the test

        :return:
        :rtype:
        """

        self.theme_hook = auth_hooks.TerraNanotechThemeHook()

    def test_should_have_correct_name_and_description(self):
        """
        Test should have the correct name and description

        :return:
        :rtype:
        """

        self.assertEqual(first=self.theme_hook.name, second="Terra Nanotech")
        self.assertEqual(
            first=self.theme_hook.description, second="Terra Nanotech Corp Auth Theme"
        )

    def test_should_have_correct_number_of_css_and_js_files(self):
        """
        Test should have the correct number of css and js files

        :return:
        :rtype:
        """

        self.assertEqual(first=len(self.theme_hook.css), second=5)
        self.assertEqual(first=len(self.theme_hook.js), second=3)

    def test_should_have_correct_header_padding(self):
        """
        Test should have correct header padding

        :return:
        :rtype:
        """

        self.assertEqual(first=self.theme_hook.header_padding, second="3.6em")

    @patch("tnnt_templates.theme.terra_nanotech.auth_hooks.TerraNanotechThemeHook")
    def test_should_register_theme_hook(self, mock_theme_hook):
        """
        Test should register theme hook

        :param mock_theme_hook:
        :type mock_theme_hook:
        :return:
        :rtype:
        """

        auth_hooks.register_terra_nanotech_hook()
        mock_theme_hook.assert_called_once()



"""
App settings
"""

# Django
from django.apps import apps

# Alliance Auth (External Libs)
from app_utils.app_settings import clean_setting


class AppSettings:  # pylint: disable=too-few-public-methods
    """
    App settings from local.py
    """

    TNNT_TEMPLATE_ENTITY_TYPE = clean_setting(
        name="TNNT_TEMPLATE_ENTITY_TYPE", default_value="alliance", required_type=str
    )

    TNNT_TEMPLATE_ENTITY_ID = clean_setting(
        name="TNNT_TEMPLATE_ENTITY_ID", default_value=1, required_type=int
    )

    TNNT_TEMPLATE_ENTITY_NAME = clean_setting(
        name="TNNT_TEMPLATE_ENTITY_NAME", default_value="", required_type=str
    )

    TNNT_TEMPLATE_URLS_OWN_WEBSITES = clean_setting(
        name="TNNT_TEMPLATE_URLS_OWN_WEBSITES", default_value=[]
    )

    TNNT_TEMPLATE_URLS_OTHER_WEBSITES = clean_setting(
        name="TNNT_TEMPLATE_URLS_OTHER_WEBSITES", default_value=[]
    )

    def aagdpr_installed(self) -> bool:
        """
        Check if aagdpr is installed
        """

        return apps.is_installed("aagdpr")



# Django
from django.template import Context, Template
from django.test import TestCase, modify_settings

# AA Templates: Terra Nanotech
from tnnt_templates import __version__


class TestTnntTemplateVariables(TestCase):
    """
    Test TN-NT Template tags
    """

    def setUp(self):
        self.context = Context(
            {"content": "Lorem ipsum dolor sit amet, consectetur adipiscing elit."}
        )

    def test_should_return_content_when_string_starts_with_substring(self):
        template_to_render = Template(
            "{% load tnnt_template_tags %}"
            '{% if content|startswith:"Lorem" %}{{ content }}{% endif %}'
        )
        rendered_template = template_to_render.render(self.context)
        self.assertInHTML(
            "Lorem ipsum dolor sit amet, consectetur adipiscing elit.",
            rendered_template,
        )

    def test_should_not_return_content_when_string_does_not_start_with_substring(self):
        template_to_render = Template(
            "{% load tnnt_template_tags %}"
            '{% if content|startswith:"ipsum" %}{{ content }}{% endif %}'
        )
        rendered_template = template_to_render.render(self.context)
        self.assertNotIn(
            "Lorem ipsum dolor sit amet, consectetur adipiscing elit.",
            rendered_template,
        )

    def test_should_return_versioned_static_url(self):
        context = Context({"version": __version__})
        template_to_render = Template(
            "{% load tnnt_template_tags %}"
            "{% tnnt_static 'tnnt_templates/css/fonts.min.css' %}"
        )
        rendered_template = template_to_render.render(context)
        self.assertInHTML(
            f'/static/tnnt_templates/css/fonts.min.css?v={context["version"]}',
            rendered_template,
        )

    @modify_settings(INSTALLED_APPS={"append": "aagdpr"})
    def test_should_return_true_when_app_is_installed(self):
        template_to_render = Template(
            "{% load tnnt_template_tags %}"
            '{% is_app_installed "aagdpr" as is_myapp_installed %}'
            "{% if is_myapp_installed %}True{% else %}False{% endif %}"
        )
        rendered_template = template_to_render.render(Context({"version": __version__}))
        self.assertEqual(rendered_template, "True")

    @modify_settings(INSTALLED_APPS={"remove": "aagdpr"})
    def test_should_return_false_when_app_is_not_installed(self):
        template_to_render = Template(
            "{% load tnnt_template_tags %}"
            '{% is_app_installed "aagdpr" as is_myapp_installed %}'
            "{% if is_myapp_installed %}True{% else %}False{% endif %}"
        )
        rendered_template = template_to_render.render(Context({"version": __version__}))
        self.assertEqual(rendered_template, "False")



"""
Tests for the app settings
"""

# Standard Library
from unittest.mock import patch

# Django
from django.test import TestCase, modify_settings

# AA Templates: Terra Nanotech
from tnnt_templates.app_settings import AppSettings


class TestAppSettings(TestCase):
    """
    Test the app settings
    """

    def setUp(self):
        """
        Setup tests

        :return:
        :rtype:
        """

        self.app_settings = AppSettings()

    @modify_settings(INSTALLED_APPS={"append": "aagdpr"})
    def test_aagdpr_installed(self):
        """
        Test if the aagdpr app is installed

        :return:
        :rtype:
        """

        # app_settings = AppSettings()

        self.assertEqual(first=self.app_settings.aagdpr_installed(), second=True)

    @modify_settings(INSTALLED_APPS={"remove": "aagdpr"})
    def test_aagdpr_not_installed(self):
        """
        Test if the aagdpr app is not installed

        :return:
        :rtype:
        """

        self.assertEqual(first=self.app_settings.aagdpr_installed(), second=False)

    def test_TNNT_TEMPLATE_ENTITY_TYPE_default_value(self):
        """
        Test the default value of TNNT_TEMPLATE_ENTITY_TYPE

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_ENTITY_TYPE, second="alliance"
        )

    @patch.object(
        target=AppSettings, attribute="TNNT_TEMPLATE_ENTITY_TYPE", new="corporation"
    )
    def test_TNNT_TEMPLATE_ENTITY_TYPE_corporation(self):
        """
        Test the value of TNNT_TEMPLATE_ENTITY_TYPE when set to "corporation"

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_ENTITY_TYPE, second="corporation"
        )

    def should_return_default_TNNT_TEMPLATE_ENTITY_ID(self):
        """
        Test the default value of TNNT_TEMPLATE_ENTITY_ID

        :return:
        :rtype:
        """

        self.assertEqual(first=self.app_settings.TNNT_TEMPLATE_ENTITY_ID, second=1)

    @patch.object(target=AppSettings, attribute="TNNT_TEMPLATE_ENTITY_ID", new=2)
    def should_return_custom_TNNT_TEMPLATE_ENTITY_ID(self):
        """
        Test the custom value of TNNT_TEMPLATE_ENTITY_ID

        :return:
        :rtype:
        """

        self.assertEqual(first=self.app_settings.TNNT_TEMPLATE_ENTITY_ID, second=2)

    def should_return_default_TNNT_TEMPLATE_ENTITY_NAME(self):
        """
        Test the default value of TNNT_TEMPLATE_ENTITY_NAME

        :return:
        :rtype:
        """

        self.assertEqual(first=self.app_settings.TNNT_TEMPLATE_ENTITY_NAME, second="")

    @patch.object(
        target=AppSettings, attribute="TNNT_TEMPLATE_ENTITY_NAME", new="Test Name"
    )
    def should_return_custom_TNNT_TEMPLATE_ENTITY_NAME(self):
        """
        Test the custom value of TNNT_TEMPLATE_ENTITY_NAME

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_ENTITY_NAME, second="Test Name"
        )

    def should_return_default_TNNT_TEMPLATE_URLS_OWN_WEBSITES(self):
        """
        Test the default value of TNNT_TEMPLATE_URLS_OWN_WEBSITES

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_URLS_OWN_WEBSITES, second=[]
        )

    @patch.object(
        target=AppSettings,
        attribute="TNNT_TEMPLATE_URLS_OWN_WEBSITES",
        new=["http://example.com"],
    )
    def should_return_custom_TNNT_TEMPLATE_URLS_OWN_WEBSITES(self):
        """
        Test the custom value of TNNT_TEMPLATE_URLS_OWN_WEBSITES

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_URLS_OWN_WEBSITES,
            second=["http://example.com"],
        )

    def should_return_default_TNNT_TEMPLATE_URLS_OTHER_WEBSITES(self):
        """
        Test the default value of TNNT_TEMPLATE_URLS_OTHER_WEBSITES

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_URLS_OTHER_WEBSITES, second=[]
        )

    @patch.object(
        target=AppSettings,
        attribute="TNNT_TEMPLATE_URLS_OTHER_WEBSITES",
        new=["http://example.com"],
    )
    def should_return_custom_TNNT_TEMPLATE_URLS_OTHER_WEBSITES(self):
        """
        Test the custom value of TNNT_TEMPLATE_URLS_OTHER_WEBSITES

        :return:
        :rtype:
        """

        self.assertEqual(
            first=self.app_settings.TNNT_TEMPLATE_URLS_OTHER_WEBSITES,
            second=["http://example.com"],
        )



"""
Tests for the context processors
"""

# Standard Library
from unittest.mock import patch

# Django
from django.http import HttpRequest
from django.test import TestCase

# AA Templates: Terra Nanotech
from tnnt_templates.app_settings import AppSettings
from tnnt_templates.context_processors import tnnt_settings


class ContextProcessorsTests(TestCase):
    """
    Test the context processors
    """

    def setUp(self):
        """
        Setup tests

        :return:
        :rtype:
        """

        self.request = HttpRequest()

    def test_should_return_default_entity_id_when_not_set(self):
        """
        Test the default value of TNNT_TEMPLATE_ENTITY_ID

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_ENTITY_ID"], second=1)

    @patch.object(target=AppSettings, attribute="TNNT_TEMPLATE_ENTITY_ID", new=5)
    def test_should_return_custom_entity_id_when_set(self):
        """
        Test the custom value of TNNT_TEMPLATE_ENTITY_ID

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_ENTITY_ID"], second=5)

    def test_should_return_alliance_as_default_entity_type_when_not_set(self):
        """
        Test the default value of TNNT_TEMPLATE_ENTITY_TYPE

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_ENTITY_TYPE"], second="alliance")

    @patch.object(
        target=AppSettings, attribute="TNNT_TEMPLATE_ENTITY_TYPE", new="corporation"
    )
    def test_should_return_custom_entity_type_when_set(self):
        """
        Test the custom value of TNNT_TEMPLATE_ENTITY_TYPE

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(
            first=result["TNNT_TEMPLATE_ENTITY_TYPE"], second="corporation"
        )

    def test_should_return_empty_string_as_default_entity_name_when_not_set(self):
        """
        Test the default value of TNNT_TEMPLATE_ENTITY_NAME

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_ENTITY_NAME"], second="")

    @patch.object(
        target=AppSettings, attribute="TNNT_TEMPLATE_ENTITY_NAME", new="Starfleet"
    )
    def test_should_return_custom_entity_name_when_set(self):
        """
        Test the custom value of TNNT_TEMPLATE_ENTITY_NAME

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_ENTITY_NAME"], second="Starfleet")

    def test_should_not_include_own_websites_when_not_set(self):
        """
        Test the default value of TNNT_TEMPLATE_URLS_OWN_WEBSITES

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_URLS_OWN_WEBSITES"], second=[])

    @patch.object(
        target=AppSettings,
        attribute="TNNT_TEMPLATE_URLS_OWN_WEBSITES",
        new=["http://example.com"],
    )
    def test_should_include_custom_own_websites_when_set(self):
        """
        Test the custom value of TNNT_TEMPLATE_URLS_OWN_WEBSITES

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(
            first=result["TNNT_TEMPLATE_URLS_OWN_WEBSITES"],
            second=["http://example.com"],
        )

    def test_should_not_include_other_websites_when_not_set(self):
        """
        Test the default value of TNNT_TEMPLATE_URLS_OTHER_WEBSITES

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(first=result["TNNT_TEMPLATE_URLS_OTHER_WEBSITES"], second=[])

    @patch.object(
        target=AppSettings,
        attribute="TNNT_TEMPLATE_URLS_OTHER_WEBSITES",
        new=["http://example.com"],
    )
    def test_should_include_custom_other_websites_when_set(self):
        """
        Test the custom value of TNNT_TEMPLATE_URLS_OTHER_WEBSITES

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(
            first=result["TNNT_TEMPLATE_URLS_OTHER_WEBSITES"],
            second=["http://example.com"],
        )

    def test_should_return_the_aa_logo(self):
        """
        Test the default value of TNNT_TEMPLATE_AA_LOGO

        :return:
        :rtype:
        """

        result = tnnt_settings(request=self.request)

        self.assertEqual(
            first=result["TNNT_TEMPLATE_AA_LOGO"],
            second="/static/allianceauth/images/auth-logo.svg",
        )



"""
AppConfig for Terra Nanotech theme.
"""

# Django
from django.apps import AppConfig


class TerraNanotechThemeConfig(AppConfig):
    """
    AppConfig for Terra Nanotech theme.
    """

    name = "tnnt_templates.theme.terra_nanotech"
    label = "terra_nanotech"
    version = "5.3.2"
    verbose_name = f"Terra Nanotech v{version}"

    def ready(self):
        """
        Ready

        :return:
        :rtype:
        """

        pass  # pylint: disable=unnecessary-pass



"""
TN-NT Templates content processor
"""

# Django
from django.core.handlers.wsgi import WSGIRequest

# AA Templates: Terra Nanotech
from tnnt_templates.app_settings import AppSettings


def tnnt_settings(request: WSGIRequest) -> dict:  # pylint: disable=unused-argument
    """
    Returning a settings dict
    :param request:
    :return:
    """

    # AA logo
    return_value = {
        "TNNT_TEMPLATE_AA_LOGO": "/static/allianceauth/images/auth-logo.svg"
    }

    # entity ID
    # 1 if none is given
    try:
        return_value["TNNT_TEMPLATE_ENTITY_ID"] = AppSettings.TNNT_TEMPLATE_ENTITY_ID
    except AttributeError:
        return_value["TNNT_TEMPLATE_ENTITY_ID"] = 1

    # entity type
    # 'alliance' if none is given
    try:
        return_value["TNNT_TEMPLATE_ENTITY_TYPE"] = (
            AppSettings.TNNT_TEMPLATE_ENTITY_TYPE
        )
    except AttributeError:
        return_value["TNNT_TEMPLATE_ENTITY_TYPE"] = "alliance"

    # entity name
    # empty if none is given
    try:
        return_value["TNNT_TEMPLATE_ENTITY_NAME"] = (
            AppSettings.TNNT_TEMPLATE_ENTITY_NAME
        )
    except AttributeError:
        return_value["TNNT_TEMPLATE_ENTITY_NAME"] = ""

    try:
        return_value["TNNT_TEMPLATE_URLS_OWN_WEBSITES"] = (
            AppSettings.TNNT_TEMPLATE_URLS_OWN_WEBSITES
        )
    except AttributeError:
        pass

    try:
        return_value["TNNT_TEMPLATE_URLS_OTHER_WEBSITES"] = (
            AppSettings.TNNT_TEMPLATE_URLS_OTHER_WEBSITES
        )
    except AttributeError:
        pass

    return return_value



"""
Auth hooks for Terra Nanotech theme
"""

# Standard Library
from urllib.parse import urljoin

# Django
from django.conf import settings

# Alliance Auth
from allianceauth import hooks
from allianceauth.theme.hooks import ThemeHook


class TerraNanotechThemeHook(ThemeHook):
    """
    Terra Nanotech Corp Auth Theme
    https://github.com/terra-nanotech/tn-nt-auth-templates/
    """

    def __init__(self):
        """
        Init
        """

        ThemeHook.__init__(
            self,
            "Terra Nanotech",
            "Terra Nanotech Corp Auth Theme",
            css=[
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/libs/bootstrap/v5.3.3/css/bootstrap.min.css",
                    ),
                    "integrity": "sha512-lF+xS8uroqRohnQyVXSTrsB+YgYcwHKVm8T6atFzc/cnOW1RTnc6x00585jS74sz9GPrNbzH4QkP8JICSXNP0Q==",
                },
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/css/terra-nanotech-defaults.min.css",
                    ),
                    "integrity": "sha512-E8vMqXFN+EiyBKkt06o/x5rK+ZGQk8f8+jgWJX5ZKDoxjG4noss9lfS/cgp5/evqXRlmdhBT8IB8QToCPNbq3w==",
                },
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/css/terra-nanotech-fonts.min.css",
                    ),
                    "integrity": "sha512-mF/lpSPp8ihS9XOtcOPAFzpzsQRMKH3D0scCPEKIaTpVtDd7F6iIafkdkD1VJ1ubsQKy3CUiBWigoVKE62TlXQ==",
                },
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/css/terra-nanotech.min.css",
                    ),
                    "integrity": "sha512-/M3Z2UIOXMWwRF23o7n7bLgCXL5z3CBPJY4BEHHTP7uTAecPTCbt615LVZIU01nLzhkqEAOyN0V4TPJEUgacgA==",
                },
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/css/community-app-fixes.min.css",
                    ),
                    "integrity": "sha512-iOn30VaTPjiuGy74s2p+0YKs8URpvxXJVnZfcyMz6U1CD463h8hK5xS8QKuBqmeCJ9ECQ7HNlamn8rfGCfeU0Q==",
                },
            ],
            js=[
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/libs/popper/v2.11.8/popper.min.js",
                    ),
                    "integrity": "sha512-fhcY1KxngNJ4jVhZBdmrLv4/NH31jrNM45fpxytokYp06cd7Ug05E3gximUQmukhES9Xf0kbV5F1nHVqWq2bvQ==",
                },
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/theme/terra-nanotech/aav4/libs/bootstrap/v5.3.3/javascript/bootstrap.min.js",
                    ),
                    "integrity": "sha512-gNyiMtmOs5iIO2fjMFZRSR1s9Espoi+fdDtNuSh1iMpeRminsho2AA7767qpfkYqskd9PtUfMwAg0KdKJsMTuQ==",
                },
                {
                    "url": urljoin(
                        settings.STATIC_URL,
                        "tnnt_templates/javascript/terra-nanotech.min.js",
                    ),
                    "integrity": "sha512-8vfSXueHbC/o6MgFxuPAEbbc9jMObN+r+rnk6uYDEYVdzJdCcOWglSoTmaPuGn5xurevBMnFY1IEWWpzcD31Lg==",
                },
            ],
            header_padding="3.6em",
        )


@hooks.register("theme_hook")
def register_terra_nanotech_hook():
    """
    Registers the Terra Nanotech theme hook

    :return:
    :rtype:
    """

    return TerraNanotechThemeHook()
